@@ -321,8 +321,13 @@ get_download.default <- function(x, verbose = TRUE) {
             
             chrons <- try(unique(as.vector(unlist(chron_vectors))), silent = TRUE)
 
-            base.frame <- as.data.frame(matrix(ncol = 7,
-                                               nrow = nrow(sample.meta)))
+            base.frame <- data.frame(age.older = rep(NA, nrow(sample.meta)),
+                                     age = rep(NA, nrow(sample.meta)),
+                                     age.younger = rep(NA, nrow(sample.meta)),
+                                     chronology.name = rep(NA, nrow(sample.meta)),
+                                     age.type = rep(NA, nrow(sample.meta)), 
+                                     chronology.id = rep(NA, nrow(sample.meta)), 
+                                     dataset.id = rep(NA, nrow(sample.meta)))
             
             colnames(base.frame) <- c('age.older', 'age',
                                       'age.younger', 'chronology.name',
@@ -445,7 +450,7 @@ get_download.default <- function(x, verbose = TRUE) {
             cast_table <- reshape2::dcast(sample.data, 
                                 taxon.name + variable.units + variable.element +
                                   variable.context + taxon.group +
-                                  ecological.group ~ sample.id, value.var = "value")
+                                  ecological.group ~ sample.id, value.var = "value", fun.aggregate = sum)
             
             taxon.list <- cast_table[ ,1:6]
             
